<?php
/**
 * @package Apple
 * @author Parsiserve Inc.
 * @copyright Parsiserve Inc. http://www.parsiserve.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see license.txt
 * Apple is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; version 2 of the License.
 */

defined('ACCESS_ALLOW') or die('<H1>Forbidden</H1>You are not allowed to be here<hr>');

class fileManager {
	static private $instance = null;
	private $rootPath, $url;
	private $path, $file, $up;
	private $absolutePath;
	private $directories = array();
	private $files = array();
	
	function __construct($rootPath=null, $url=null) {
		$this->rootPath = ($rootPath == null) ? Apple_root_path.'/uploads' : $rootPath;
		$this->url = ($url == null) ? settings::get('url').'/uploads' : $url;
		
		$this->path = AppleGetParam('path', $_GET);
		$this->file = AppleGetParam('file', $_GET);
		$path = explode('/', $this->path);
		if (!$this->securePath($path)) die('Attack found!');
		$this->path = $this->trimPath($path);
		$this->absolutePath = $this->rootPath.'/'.$this->path;
	}
	
	static public function getInstance() {
		if (self::$instance == null) self::$instance = new fileManager();
		return self::$instance;
	}
	
	function view() {
		$this->fd();
		$display = new filesHTML();
		$display->view();
		//echo $this->path;
		//rename($this->absolutePath.'/as', $this->absolutePath.'/AS');
	}
	
	function fd() {
		$dir = @dir($this->absolutePath);
		if ($dir == null) {
			echo __('Root directory not found');
			return;
		}
		
		while (($file = $dir->read()) != null) {
			if (is_file($this->absolutePath.'/'.$file)) array_push($this->files, $file);
			elseif ($file != '.' && $file != '..') array_push($this->directories, $file);
		}
	}
	
	/**
	 * gets directories, files list that created before
	 *
	 * @return Array of Array
	 */
	function getList() {
		return array($this->directories, $this->files);
	}
	
	/**
	 * gets attributes (such as private attributes)
	 *
	 * @param string $name
	 * @return string attribute
	 */
	function get($name) {
		return $this->$name;
	}
	
	/**
	 * Trims the path
	 *
	 * @param Array $path
	 * @return string
	 */
	function trimPath($path) {
		$stack = array();
		foreach ($path as $file) {
			if ($file != null) {
				array_push($stack, $file);
			}
		}
		
		$path = null;
		$count = count($stack);
		for ($i=0; $i<$count; $i++) {
			$file = $stack[$i];
			if ($path != null) $path .= '/';
			$path .= $file;
			if ($i == $count-2) $this->up = '| <a href="javascript:void(0)" onclick="Clear(\'fm_details\'); Clear(\'operations\'); go(\'?com=files&task=go&path='.$path.'&load_theme=no&lng='.LANGUAGE.'\', \'fm_list\')">'.__('Back').' <img src="images/back.png" style="vertical-align: middle;" alt=""></a>';
		}
		if ($this->up == null && $path != null) $this->up = '| <a href="javascript:void(0)" onclick="Clear(\'fm_details\'); Clear(\'operations\'); go(\'?com=files&task=go&load_theme=no&lng='.LANGUAGE.'\', \'fm_list\')">'.__('Back').' <img src="images/back.png" style="vertical-align: middle;" alt=""></a>';
		return $path;
	}
	
	/**
	 * checks path for security
	 *
	 * @param Array $path
	 * @return FALSE if found bad name such as '..' ;-)
	 */
	function securePath($path) {
		foreach($path as $name) {
			if ($name == '..') return false;
		}
		return true;
	}
	
	function viewList() {
		$this->fd();
		filesHTML::viewList();
	}
	
	function details() {
		$type = filetype($this->absolutePath.'/'.$this->file);
		filesHTML::details($type);
	}
	
	function refresh($path=null) {
		if ($path == null) $path = $this->path;
		echo '<script type="text/javascript">Clear(\'fm_details\'); go(\'?com=files&path='.$path.'&task=go&load_theme=no&lng='.LANGUAGE.'\', \'fm_list\');</script>';
	}
	
	function sideForm() {
		$form = AppleGetParam('form', $_GET, 'create_folder');
		$type = filetype($this->absolutePath.'/'.$this->file);
		
		switch ($form) {
			case 'create_folder' : $this->createFolder(); break;
			case 'create_file' : $this->createFile(); break;
			case 'rename' : $this->rename($type); break;
			case 'delete' : $this->delete($type); break;
			case 'upload' : $this->upload(); break;
		}
	}
	
	function createFolder() {
		$name = AppleGetParam('name', $_POST, null);
		if ($name == null) filesHTML::createFolder();
		else {
			if (file_exists($this->absolutePath.'/'.$name)) {
				printf(AppleShowMessage(__('Folder %s exists.'), 1, 2), $name);
				return ;
			}
			if (mkdir($this->absolutePath.'/'.$name, 0777)) printf(AppleShowMessage(__('Folder %s has been created successfully.'), 0, 2), $name);
			else printf(AppleShowMessage(__('Folder %s not created.'), 1, 2), $name);
			$this->refresh();
		}
	}
	
	function createFile() {
		$name = AppleGetParam('name', $_POST, null);
		if ($name == null) filesHTML::createFile();
		else {
			if (file_exists($this->absolutePath.'/'.$name)) {
				printf(AppleShowMessage(__('File %s exists'), 1, 2), $name);
				return ;
			}
			
			if (fopen($this->absolutePath.'/'.$name, 'w')) printf(AppleShowMessage(__('File %s created successfully'), 0, 2), $name);
			else printf(AppleShowMessage(__('File %s not created'), 1, 2), $name);
			$this->refresh();
		}
	}
	
	function rename($type) {
		$name = AppleGetParam('name', $_POST, null);
		
		if ($name == null) filesHTML::rename($type);
		else {
			if (!file_exists($this->absolutePath.'/'.$this->file)) {
				echo $this->absolutePath.'/'.$this->file;
				$type == 'file' ? printf(AppleShowMessage(__('File %s not exists'), 1, 2), $this->file) : printf(AppleShowMessage(__('Folder %s not exists.'), 1, 2), $this->file);
				return ;
			}
			elseif ($this->file == $name) return ;
			elseif (file_exists($this->absolutePath.'/'.$name)) echo ($type == 'file') ? AppleShowMessage(__('File with the same name exists'), 1, 2) : AppleShowMessage(__('Folder with the same name exists.'), 1, 2);
			elseif (rename($this->absolutePath.'/'.$this->file, $this->absolutePath.'/'.$name)) {
				printf(AppleShowMessage(__('%s renamed to %s successfully'), 0, 2), $this->file, $name);
				$this->refresh();
			}
			else printf(AppleShowMessage(__('%s not renamed'), 1, 2), $this->file);
		}
	}
	
	function delete($type='dir') {
		if (!file_exists($this->absolutePath.'/'.$this->file)) {
				$type == 'file' ? printf(AppleShowMessage(__('File %s not exists'), 1, 2), $this->file) : printf(AppleShowMessage(__('Folder %s not exists.'), 1, 2), $this->file);
				return ;
		}
		
		if ($type == 'file') {
			if (unlink($this->absolutePath.'/'.$this->file)) {
				printf(AppleShowMessage(__('%s deleted successfully'), 0, 2), $this->file);
				$this->refresh();
			}
			else printf(AppleShowMessage(__('%s not deleted'), 1, 2), $this->file);
		}
		else {
			if (!$this->delete2($this->absolutePath.'/'.$this->file)) printf(AppleShowMessage(__('%s not deleted'), 1, 2), $this->file);
			else {
				printf(AppleShowMessage(__('%s deleted successfully'), 0, 2), $this->file);
				$this->refresh();
			}
		}
	}
	
	function delete2($dirName) {
		@$dir=opendir("$dirName");
		if (!$dir) return false;
		while($file=readdir($dir)) {
			if ($file != '.' && $file != '..' && is_dir("$dirName/$file")) {
				$this->delete2("$dirName/$file");
			}
			else if ($file != '.' && $file != '..' && is_file("$dirName/$file")) {
				unlink ("$dirName/$file");
			}
		}
		closedir($dir);
		rmdir("$dirName");
		return true;
	}
	
	function upload() {
		$submit = AppleGetParam('submit', $_POST, null);
		if ($submit == null) filesHTML::upload();
		else {
			$tmpName = $_FILES['f_1']['tmp_name'];
			$fileName = $_FILES['f_1']['name'];
			$over = AppleGetParam('over', $_POST, 'no');
			if ($over == 'no' && file_exists($this->absolutePath.'/'.$fileName)) echo '<div style="text-align:right;">'.AppleShowMessage(__('File with the same name exists'), 1, 2).'</div>';
			elseif (copy($tmpName, $this->absolutePath.'/'.$fileName)) {
				echo '<div style="text-align:right;">'.AppleShowMessage(__('File uploaded successfully'), 0, 2).'</div>
				<script type="text/javascript">parent.go(\'?com=files&task=go&path='.$this->path.'&load_theme=no&lng='.LANGUAGE.'\', \'fm_list\');</script>';
			}
			else echo '<div style="text-align:right;">'.AppleShowMessage(__('unknown_error'), 1, 2).'</div>';
		}
	}
}
?>