<?php
/**
 * @package Apple
 * @author Parsiserve Inc.
 * @copyright Parsiserve Inc. http://www.parsiserve.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see license.txt
 * Apple is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; version 2 of the License.
 */

defined('ACCESS_ALLOW') or die('<H1>Forbidden</H1>You are not allowed to be here<hr>');

/**
 * database class
 */

class database {
    private $host=null, $user=null, $password=null;
	private $database=null;
	private $conn=null, $result=false;
	private $sql=array(), $sql_count = 0;
	public  $prefix;
	private $cursor;
	private $error_num=0, $error_msg=null;
	
	public function __construct($host, $user, $password, $database, $prefix) {
		$this->host = $host;
		$this->user = $user;
		$this->password = $password;
		$this->database = $database;
		$this->prefix = $prefix;
		return self::open();
	}
	
	/**
	 * Open Connection & Select database
	*/
	protected function open() {
		if(!$this->conn = @mysql_connect($this->host, $this->user, $this->password)) {
			AppleDie(sprintf(/* This message AND suggestions are from WordPress */"<h1>Error establishing a database connection</h1>
			<p>This either means that the username and password information in your <code>configuration.php</code> file is incorrect or we can't contact the database server at <code>%s</code>. This could mean your host's database server is down.</p>
			<ul>
				<li>Are you sure you have the correct username and password?</li>
				<li>Are you sure that you have typed the correct hostname?</li>
				<li>Are you sure that the database server is running?</li>
			</ul>
			<p>If you're unsure what these terms mean you should probably contact your host. If you still need help you can always visit the <a href='http://forums.applecms.org' target='_blank'>Apple Forums</a>.</p>", $this->host));
		}
		if(!@mysql_select_db($this->database, $this->conn)) {
			AppleDie(sprintf(/* This message AND suggestions are from WordPress */'<h1>Can\'t select database</h1>
			<p>We were able to connect to the database server (which means your username and password is okay) but not able to select the <code>%1$s</code> database.</p>
			<ul>
				<li>Are you sure it exists?</li>
				<li>Does the user <code>%2$s</code> have permission to use the <code>%1$s</code> database?</li>
				<li>On some systems the name of your database is prefixed with your username, so it would be like <code>username_%1$s</code>. Could that be the problem?</li>
			</ul>
			<p>If you don\'t know how to setup a database you should <strong>contact your host</strong>. If all else fails you may find help at the <a href="http://forums.applecms.org" target="_blank">Apple Forums</a>.</p>', $this->database, $this->user));
		}
		return true;
	}
	
	/**
	 * Close Connection
	*/
	protected function close() {
	    return(@mysql_close($this->conn));
	}
	
	/**
	* Sets the SQL query string for later execution
	*
	* @param string The SQL query
	* @param string The common table prefix
	*/
	public function setQuery($sql, $prefix='#__') {
		$this->sql[$this->sql_count++] = $this->replacePrefix($sql, $prefix);
	}
	
	/**
	* Execute the query (queries)
	* @return database cursor if successful, false if not.
	*/
	public function query($sql=null) {
		if ($sql != null) $this->sql[$this->sql_count++]=$sql;
		while ($this->sql != null) {
			if ($this->cursor = mysql_query(array_shift($this->sql), $this->conn)) {
				$this->error_num = 0;
				$this->error_msg = null;
				$this->sql_count--;
			}
			else {
				$this->sql = array();
				$this->sql_count = 0;
				$this->error_num = mysql_errno( $this->conn);
				$this->error_msg = mysql_error( $this->conn)." SQL=$sql";
				return false;
			}
		}
		$this->sql = array();
		$this->sql_count = 0;
		return $this->cursor;
	}
	
	/**
	 * Load the first row of the query
	 *
	 * @return object
	 */
	public function loadRow($type='row') {
		$result = $this->retrieveResults(null, 1, $type);
		if (count($result)) return $result[0];
		return null;
	}
	
	/**
	* Load a list of database rows
	* @param string The field name of a primary key
	* @return array If <var>key</var> is empty as sequential list of returned records.
	* If <var>key</var> is not empty then the returned array is indexed by the value
	* the database key.  Returns <var>null</var> if the query fails.
	* 
	*/
	public function loadRowList($key=null, $type='row') {
		$results = $this->retrieveResults($key, 0, $type);
		if (count($results)) return $results;
		return null;
	}
	
	/**
	 * Loads the first row of a query into an object
	 *
	 * @return object
	 */
	public function loadObject() {
		return $this->loadRow('object');
	}
	
	/**
	* Load a list of database objects
	* @param string The field name of a primary key
	* @return array If <var>key</var> is empty as sequential list of returned records.
	* If <var>key</var> is not empty then the returned array is indexed by the value
	* the database key.  Returns <var>null</var> if the query fails.
	* 
	*/
	public function loadObjectList($key=null) {
		return $this->loadRowList($key, 'object');
	}
	
	/**
	* Load an array of retrieved database objects or values
	* @param int Database cursor
	* @param string The field name of a primary key
	* @return array If <var>key</var> is empty as sequential list of returned records.
	* If <var>key</var> is not empty then the returned array is indexed by the value
	* the database key.  Returns <var>null</var> if the query fails.
	* @copyright Mambo's database class
	*/
	protected function &retrieveResults($key=null, $max=0, $result_type='row') {
		$results=array();
		$sql_function='mysql_fetch_'.$result_type;
		if($cur=$this->query()){
			while($row=$sql_function($cur)){
				if($key != null){
					if( is_array($row) ){
						$results[$row[$key]]=$row;
					} else{
						$results[$row->$key]=$row;
					}
				} else{
					$results[]=$row;
				}
				if($max AND count($results) >= $max) break;
			}
			mysql_free_result($cur);
		}
		return $results;
	}
	
	/**
	* @return int The error number
	*/
	public function getErrorNum() {
		return $this->error_num;
	}
	
	/**
	* @return string The error message
	*/
	public function getErrorMsg() {
		return $this->error_msg;
	}
	
	/**
	 * This function replaces a string identifier <var>$prefix</var> with the
	 * string held is the <var>prefix</var> class variable.
	 *
	 * @param string The SQL query
	 * @param string The common table prefix
	 * @return string The SQL query
	 */
	protected function replacePrefix($sql, $prefix='#__') {
		$done = null;
		$single = false;
		$double = false;
		$found = false;
		$i = 0;
		while (strlen($sql) > 0) {
			if ($sql[$i] == null) return $done.$sql;
			if (($sql[$i] == "'") && $sql[$i-1] !="\\")
				$single = !$single;
			if (($sql[$i] == '"') && $sql[$i-1] !="\\")
				$double = !$double;
			
			if ($sql[$i] == $prefix[0] && !$single && !$double) {
				$found = true;
				for ($j=0; $j < strlen($prefix); $j++) {
					if ($sql[$i+$j] != $prefix[$j]) {
						$found = false;
						break;
					}
				}
			}
			
			if ($found) {
				$done .= substr($sql, 0, $i).$this->prefix;
				$sql = substr($sql, $i+$j);
				$found = false;
				$i = 0;
			}
			else
				$i++;
			//if ($i >= strlen($sql)) return $done.$sql;
		}
		return $done;
	}
	
	/**
	 *
	 * @return int last id that inserted
	 */
	public function insertId() {
		return mysql_insert_id();
	}
	
	public function getAffectedRows() {
		return mysql_affected_rows( $this->_resource );
	}
	
	/**
	 * inserts the object into database
	 *
	 * @param string $table
	 * @param object $object
	 * @param array $ignore
	 * @return int id if insert successfully otherwise return 0
	 */
	public function insert($table, $object, $ignore=null) {
		$sql = "INSERT INTO $table (%s) VALUES (%s) ;";
		$sql = $this->replacePrefix($sql);
		$ignore = explode(',', $ignore);
		$fields = array();
		$values = array();
		if (is_array($ignore)) {
			for ($i=0; $i<count($ignore); $i++) {
				$ignore[$i] = trim($ignore[$i]);
			}
		}
		foreach (get_object_vars($object) as $k => $v) {
			if (in_array($k, $ignore)) continue;
			$fields[] = "`$k`";
			$values[] = "'$v'";
		}
		$sql = sprintf($sql, implode($fields, ','), implode($values, ','));
		$this->sql[$this->sql_count++] = $sql;
		if (!$this->query()) return false;
		return $this->insertId() > 0 ? $this->insertId() : true;
	}
	
	public function update($table, $object, $where) {
		$sql = "UPDATE $table SET %s WHERE $where";
		$sql = $this->replacePrefix($sql);
		$fields = array();
		foreach (get_object_vars($object) as $k => $v) {
			$fields[] = "`$k`='$v'";
		}
		$sql = sprintf($sql, implode($fields, ','));
		$this->sql[$this->sql_count++] = $sql;
		return $this->query();
	}
	
	/**
	 * removes the object from database
	 *
	 * @param string $table
	 * @param string $where
	 * @return bool
	 */
	public function delete($table, $where) {
		$sql = "DELETE FROM $table WHERE $where";
		$sql = $this->replacePrefix($sql);
		return $this->query($sql);
	}
	
	public function sort($table, $id, $do, $fields=null, $ordering='ordering') {
		$this->setQuery("SELECT * FROM $table WHERE id=$id");
		$ob1 = $this->loadObject();
		
		$where = null;
		if ($fields != null) {
			$fields = explode(',', $fields);
			foreach ($fields as $field) {
				$field = trim($field);
				if ($field == null) continue;
				$where .= ' AND '.$field.'=\''.$ob1->$field.'\'';
				}
			}
		
		if ($do == 'up') $this->setQuery("SELECT * FROM $table WHERE id!=$id AND $ordering<=".$ob1->$ordering." $where ORDER BY $ordering");
		else $this->setQuery("SELECT * FROM $table WHERE id!=$id AND $ordering>=".$ob1->$ordering." $where ORDER BY $ordering LIMIT 1");
		
		$ob2 = $this->loadObjectList();
		$ob2 = $ob2[count($ob2)-1];
		
		if ($ob2 == null) return false; //NO operation
		
		$or1 = $ob1->$ordering;
		$or2 = $ob2->$ordering;
		
		if ($or1 != $or2) { //Exchange them
			$or1 = $ob2->$ordering;
			$or2 = $ob1->$ordering;
		}
		else {
			if ($do == 'up') {
				if ($ob1->$ordering != 1) $or1 = $ob1->$ordering-1;
				else $or2 = $ob2->$ordering+1;
			}
			else $or1 = $ob1->$ordering+1;
		}
		
		$this->setQuery("UPDATE $table SET `$ordering`='$or1' WHERE id='".$ob1->id."'");
		$this->setQuery("UPDATE $table SET `$ordering`='$or2' WHERE id='".$ob2->id."'");
		return $this->query();
	}
	
	/**
	 * import SQL from file and split SQLs with getQuery()
	 *
	 * @param string $file
	 */
	public function importSql($file) {
		$query = fread(fopen($file, 'r'), filesize($file));
		$sql = $this->getQuery($query);
		
		for ($i=0; $i<count($sql); $i++) {
			if(!empty($sql[$i]))
				$this->setQuery($sql[$i]);
		}
		return $this->query();
	}
	
	public function getQuery($sql) {
		$sql = trim($sql);
		$done = array();
		$quote = false;
		$comment = false;
		$c = 0;
		
		for($i=0; $i<strlen($sql); $i++) {
			if (($sql[$i] == "'") && $sql[$i-1] !="\\")
				$quote = !$quote;
			if (($sql[$i] == '#' || $sql[$i] == '-') && $sql[$i-1] == "\n" && !$quote)
				$comment = true;
			if ($sql[$i] == "\n" && $comment == true)
				$comment = false;
			if ($sql[$i] == ';' && !$quote) {
				$done[$c] .= ';';
				$c++;
				$i++;
			}
			if (!$comment)
				$done[$c] .= $sql[$i];
		}
		return $done;
	}
}

class db extends database {
	static private $instance = null;
	
	public function __construct() {
		parent::__construct(db_host, db_user, db_password, db_name, db_prefix);
	}

	static public function getInstance() {
		if (self::$instance == null) self::$instance = new db();
		return self::$instance;
	}
}
?>