<?php
/**
 * @package Apple
 * @author Parsiserve Inc.
 * @copyright Parsiserve Inc. http://www.parsiserve.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see license.txt
 * Apple is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; version 2 of the License.
 */

/**
 * Main Apple API
 */

defined('ACCESS_ALLOW') or die('<H1>Forbidden</H1>You are not allowed to be here<hr>');

/**
 * this function shows messages
 * message AND error can be sets null so function get these from query
 * mode can be sets on 0 | 1 | 2 to show messages in different modes such as Advanced OR Text only
 *
 * @param string $message
 * @param int $error
 * @param int $mode
 * @return string
 */
function AppleShowMessage($message=null, $error=-1, $mode=0) {
	
	$message = AppleMessage::getInstance();
	return $message->getAll();
	
	/*if ($error == -1) $error=intval(AppleGetParam('error', $_GET, 0));
	if ($message == null) $message=AppleGetParam('msg', $_GET, null);
	if ($message == null) $message=AppleGetParam('message', $_GET, null);
	if ($message == null) return;
	$message = stripcslashes($message);
	switch ($error) {
		case '0' :
			return '<div class="saved">'.$message.'</div>';
			break;
		case '1' :
			return '<div class="error"><b>'.__('ERROR').' : </b>'.$message.'</div>';
			break;
	}*/
}

function AppleGetParam($name, &$arr, $def=null, $mask=0) {
	return mosGetParam($arr, $name, $def, $mask);
}

/**
* function to sanitize input values from arrays
*
* This function provides a way to sanitize inputs, should be used to obtain values from 
* _POST, _GET, _COOKIES, etc; a default value can be passed to be used in case that not 
* values are founded to the element, a binary mask can be passed to discard some of  test,
*, this value is matched with _MOS_NOTRIM, _MOS_ALLOWHTML and, _MOS_ALLOWRAW, currently
* 3 test are do it, trim, strip html and convert the value to numeric when is possible.
*
* Example of use:
*
* To get task variable from the URL and select the view like default task, you can use:
*
* <code>$task = mosGetParam ($_GET,"task","view");</code>
*
* To get task variable from the URL, select the view like default task, allows HTML and 
* without trim you can use :
*
* <code>$task = mosGetParam ($_GET,"task","view",_MOS_NOTRIM+_MOS_ALLOWHTML);</code>
*
* @acces public
* @param array &$arr reference to array which contains the value
* @param string $name name of element searched
* @param mixed $def default value to use if nothing is founded
* @param int $mask mask to select checks that will do it
* @return mixed value from the selected element or default value if nothing was found 
* @copyright Mambo 4.6.2
*/
function mosGetParam( &$arr, $name, $def=null, $mask=0 ) {
	if (!defined('_MOS_NOTRIM')) define( "_MOS_NOTRIM", 0x0001 );
	if (!defined('_MOS_ALLOWHTML')) define( "_MOS_ALLOWHTML", 0x0002 );
	if (!defined('_MOS_ALLOWRAW')) define( "_MOS_ALLOWRAW", 0x0004 );
	if (!defined('_MOS_NOMAGIC')) define( "_MOS_NOMAGIC", 0x0008 );
    if (isset( $arr[$name] )) {
        if (is_array($arr[$name])) foreach ($arr[$name] as $key=>$element) $result[$key] = mosGetParam ($arr[$name], $key, $def, $mask);
        else {
            $result = $arr[$name];
            if (!($mask&_MOS_NOTRIM)) $result = trim($result);
            if (!is_numeric( $result)) {
                if (!($mask&_MOS_ALLOWHTML)) $result = strip_tags($result);
                if (!($mask&_MOS_ALLOWRAW)) {
                    if (is_numeric($def)) $result = intval($result);
                }
            }
            if (!get_magic_quotes_gpc()) {
                $result = addslashes( $result );
            }
        }
        return $result;
    } else {
        return $def;
    }
}

function AppleClearMessage($url=null) {
	if ($url == null) $url = $_SERVER['QUERY_STRING'];
	$url = preg_replace("/(msg+(=)?+([^&]+(&)?)?)/", "", $url);
	$url = preg_replace("/(msg+(\/)?+([^\/]+(\/)?)?)/", "", $url);
	$url = preg_replace("/(message+(=)?+([^&]+(&)?)?)/", "", $url);
	$url = preg_replace("/(message+(\/)?+([^\/]+(\/)?)?)/", "", $url);
	$url = preg_replace("/((&)?msg=$msg(&)?)/", '$3', $url);
	$url = preg_replace("/((&)?message=$message(&)?)/", '$3', $url);
	return $url;
}

/**
 * Loading .mo file for translation
 *
 */
function AppleLoadMoFile($moFile, $name='system') {
	global $__phrase__;
	if (isset($__phrase__[$name]))
		return;
	if (is_readable($moFile))
		$fp = new CachedFileReader($moFile);
	else
		return;
	$__phrase__[$name] = new gettext_reader($fp);
}

/**
 * @return string translated string
 * 
 */
function __($phrase, $name='system') {
	return AppleTranslate($phrase, $name);
}

/**
 * echo translated string
 *
 */
function _e($phrase, $name='system') {
	echo AppleTranslate($phrase, $name);
}

function AppleTranslate($phrase, $name) {
	global $__phrase__;
	if (isset($__phrase__[$name]))
		return $__phrase__[$name]->translate($phrase);
	else
		return $phrase;
}

/**
 * Turn register globals off
 * @copyright WordPress Version 2.3 see http://wordpress.org
 *
 */
function wp_unregister_GLOBALS() {
	if ( !ini_get('register_globals') )
		return;

	if ( isset($_REQUEST['GLOBALS']) )
		die('GLOBALS overwrite attempt detected');

	// Variables that shouldn't be unset
	$noUnset = array('GLOBALS', '_GET', '_POST', '_COOKIE', '_REQUEST', '_SERVER', '_ENV', '_FILES', 'dbPrefix');

	$input = array_merge($_GET, $_POST, $_COOKIE, $_SERVER, $_ENV, $_FILES, isset($_SESSION) && is_array($_SESSION) ? $_SESSION : array());
	foreach ( $input as $k => $v )
		if ( !in_array($k, $noUnset) && isset($GLOBALS[$k]) ) {
			$GLOBALS[$k] = NULL;
			unset($GLOBALS[$k]);
		}
}

/**
 * this function returns date & time
 *
 * @param datetime $date
 * @param string[optional] $format
 * @param int[optional] $timestamp
 * @param int[optional] $translate
 * @return datetime
 */
function AppleDate($date, $format=null, $timestamp=0, $translate=-1, $w='L') {
	if ($date == '0000-00-00' || $date == '0000-00-00 00:00:00') return $date;
	$is_null = false;
	$date_type = settings::get('date_type');
	$date_format = settings::get('date_format');
	$time_format = settings::get('time_format');
	if ($translate == -1) $translate = settings::get('translate');
	
	if ($date == null) {
		$is_null = true;
		$date = ($timestamp != 0) ? gmdate('Y-m-d H:i:s', $timestamp) : AppleGetNow('Y-m-d H:i:s', 'L');
	}
	
	list ($date, $time) = preg_split('/ /', $date);
	list($year, $month, $day) = preg_split('/-/', $date);
	if (!empty($time)) list($h, $m, $s) = preg_split('/:/', $time);
	else $h=$m=$s='00';
	
	if (($w == 'G' || $w == 'g') && !$is_null) { // It means $date is in GMT time, so convert it
		$offset = settings::get('gmt_offset');
		list($oh, $om) = explode(':', $offset);
		$h += $oh;
		$m += $om;
	}

	if ($format == null) {
		$format = $date_format;
		if ($time != null) $format .= ' '.$time_format;
	}
	
	if ($date_type == 'jalali') return jdate($format, mktime($h,$m,$s,$month,$day,(int)$year), $translate);
	return date($format, mktime($h, $m, $s, $month, $day, $year));
}

/**
 * This function gets the date AND converts it to Gregorian date in GMT
 *
 * @param datetime date
 * @return GMT DATETIME
 */
function AppleConvertToGregorian($date, $type='jalali') {
	list($date, $time) = explode(' ', $date);
	list($year, $month, $day) = preg_split('/-/', $date);
	if ($time != null) list($hour, $minute, $second) = preg_split('/:/', $time);
	else {
		AppleGmtServer($hour, $minute);
		$second = 00;
	}
	if ($type == 'jalali') $offset = jmaketime($hour, $minute, $second, $month, $day, $year);
	$r = gmdate('Y-m-d H:i:s', $offset);
	if ($r == '1970-01-01 00:00:00') $r = '0000-00-00 00:00:00';
	return $r;
}

function AppleGetNow($format='Y-m-d H:i:s', $w='G') {
	$offset = 0;
	if ($w == 'L' || $w == 'l') { // Local date time
		$offset = settings::get('gmt_offset');
		list($h, $m) = explode(':', $offset);
		$offset = (!strstr($h, '-')) ? $h*3600+$m*60 :  $h*3600-$m*60;
	}
	return gmdate($format, time()+$offset);
}

function AppleGmtServer(&$h, &$m) {
	list($d, $h, $m) = explode('-', date('d-H-i'));
	list($gd, $gh, $gm) = explode('-', gmdate('d-H-i'));
	
	if ($d == $gd) {
		$x = (($h*60)+$m) - (($gh*60)+$gm);
	}
	elseif ($d > $gd) {
		$x = ((($h+24)*60)+$m) - (($gh*60)+$gm);
	}
	else {
		$x = (($h*60)+$m) - ((($gh+24)*60)+$gm);
	}
	
	$h = intval($x/60);
	$m = abs($x%60);
}

function AppleRedirect($url) {
	// First check if the current HTTP-Request is an Ajax-Request
	if (AppleAjaxRequest()) {
		$msg = preg_split("/(msg=)/U", $url);
		if (empty($msg[1])) $msg = preg_split("/(message=)/U", $url);
		$msg = preg_replace("/(&.*$)/U", '', $msg[1]);
		
		$error = preg_split("/(error=)/U", $url);
		$error = preg_replace("/(&.*$)/U", '', $error[1]);
		if (empty($error)) $error = 0;
		die(AppleShowMessage($msg, $error));
	}
	
	if (headers_sent()) echo '<script language="javascript">document.location.href="'.AppleMakeSef($url).'";</script>';
	else header("Location: ".AppleMakeSef($url));
	exit();
}

function AppleGetBack() {
	/*if ($_SERVER['HTTP_REFERER']) AppleRedirect(AppleClearMessage($_SERVER['HTTP_REFERER']));
	else echo '<script type="text/javascript">history.go(-1);</script>';*/
	return ;
	return $_SERVER['HTTP_REFERER'];
}

function AppleGoBack() {
	if ($_SERVER['HTTP_REFERER']) AppleRedirect(AppleClearMessage($_SERVER['HTTP_REFERER']));
	else echo '<script type="text/javascript">history.go(-1);</script>';
}

/**
 * Adds new item to path
 *
 * @param string $title
 * @param string $url
 */
function AppleSetPath($title, $url=null) {
	if (!$title) return;
	$p = ApplePath::getInstance();
	$p->addItem($title, AppleMakeSef($url));
}

function AppleGetMenuId($url) {
	global $database;
	$url = str_replace('?', null, $url);
	$parts = explode('&', $url);
	$parts[] = 'lng='.LANGUAGE;
	$where = null;
	foreach ($parts as $part) {
		$where .= "`url` LIKE '%$part%' AND ";
	}
	$where .= 'ispage=1 AND internal=1';
	
	$database->setQuery("SELECT id,url FROM `#__menu_items` WHERE $where");
	$menu = $database->loadObject();
	$menu->url = str_replace('?', null, $menu->url);
	$parts1 = explode('&', $menu->url);
	if (is_array($parts1)) {
		foreach ($parts1 as $part) {
			if (!in_array($part, $parts)) {
				unset($menu);
			}
		}
	}
	else $menu = null;
	return ($menu != null) ? $menu->id : 0;
}

function AppleGetComponentMenuId() {
	global $com, $database;
	$like = ($com == settings::get('default_component')) ? '?lng='.LANGUAGE : "%com=$com&%";
	$database->setQuery("SELECT id FROM `#__menu_items`
	WHERE url LIKE '$like' AND ispage=0 AND internal=1 AND lng='".LANGUAGE."'");
	$component_menu_id = $database->loadObject();
	return ($component_menu_id != null) ? $component_menu_id->id : 0;
}

function AppleSetMenuId($url) {
	global $Apple_menu_id;
	$Apple_menu_id = AppleGetMenuId($url);
}

function AppleSetKeywords($keywords) {
	global $Apple_theme;
	if ($Apple_theme == null) return;
	$Apple_theme->setKeywords($keywords);
}

function AppleSetTitle($title) {
	global $Apple_theme;
	if ($Apple_theme == null) return;
	$Apple_theme->setTitle($title);
}

function AppleGeneratePassword($count=8) {
	mt_srand(microtime()*1000000);
	$words = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!$';
	$return = '';
	for ($i=0; $i<$count; $i++) $return .= $words[mt_rand(0, strlen($words)-1)];
	return $return;
}

function AppleMail($from=null, $fromName=null, $recipient, $subject, $body, $html=false, $cc=null, $bcc=null, $attachment=null, $replyTo=null, $replyToName=null) {
	$mail = new AppleMailer($from, $fromName, $subject, $body);
	return $mail->mail($recipient, $html, $cc, $bcc, $attachment, $replyTo, $replyToName);
}

/**
 * Fetches objects from database AND sort
 *
 * @param string $table
 * @param integer $root
 * @param string $where
 * @param boolean $children
 * @param integer $limit_start
 * @param integer $limit_length
 * @return array
 */
function AppleMakeTree($table, $root, $where=null, $children=true, $limit_start=0, $limit_length=25) {
	$obj = new AppleMakeTree($table, $root, $where, $children, $limit_start, $limit_length);
	return $obj->list;
}

function AppleEmailValidity($email) {
	return ereg("^[^@ ]+@[^@ ]+\.[^@ \.]+$", $email);
}

/**
 * Fetch the parameters from database
 *
 * @param string $table
 * @param int $id
 * @param string $key
 * @return object
 */
function AppleGetParams($table, $id, $key=null) {
	global $database;
	$database->setQuery("SELECT * FROM $table WHERE id='$id'");
	$params = $database->loadObjectList($key);
	$object = new stdClass();
	
	if ($key == null) {
		if (!empty($params)) {
			foreach ($params as $param) {
				$key = $param->key;
				$object->param->$key = $param->value;
			}
		}
		else $object->param = null;
	}
	else $object->param = $params;
	return $object->param;
}

function AppleCaptcha($key, $width=170, $height=40) {
	$code = AppleGetParam($key, $_SESSION);
	$space_per_char = $width / (strlen($code) + 1);
	
	// Create canvas
	$img = imagecreatetruecolor($width, $height);
	
	// Allocate colors
	$background = imagecolorallocate($img, 255, 255, 255);
	$border = imagecolorallocate($img, 128, 128, 128);
	$colors[] = imagecolorallocate($img, 128, 64, 192);
	$colors[] = imagecolorallocate($img, 192, 64, 128);
	$colors[] = imagecolorallocate($img, 108, 102, 64);
	
	// Fill background
	imagefilledrectangle($img, 1, 1, $width-2, $height-2,$background);
	imagerectangle($img, 0, 0, $width - 1, $height - 1, $border);
	
	
	// Draw text
	for ($i = 0; $i < strlen($code); $i++) {
		$color = $colors[$i%count($colors)];
		imagettftext($img, $height/2 + rand(0, 8), -$height/2 + rand(0, $height), ($i+0.3)*$space_per_char, $height-8, $color, 'Times.ttf', $code[$i]);
	}
		
	for ($i = 0; $i<($width*$height/150); $i++) {
		$x1 = rand(5, $width - 5);
		$y1 = rand(5, $height - 5);
		$x2 = $x1 - 4 + rand(0, 8);
		$y2 = $y1 - 4 + rand(0, 8);
		imageline($img, $x1, $y1, $x2, $y2, $colors[rand(0, count($colors) - 1)]);
	}
	
	// Output to browser
	header('Content-type: image/png');
	imagepng($img);
}

function AppleImageResize($ffile=null, $width=0, $height=0) {
	// Get new dimensions
	list($widthorig, $heightorig) = getimagesize($ffile);
	if (!$width && !$height) {
		$width = $widthorig;
		$height = $heightorig;
	}
	if ($width && !$height) {
		$width = ($width > $widthorig) ? $widthorig : $width;
		$height = ($width > $widthorig) ? $heightorig : ($heightorig / $widthorig) * $width;
	}
	if ($height && !$width) {
		$height = ($height > $heightorig) ? $heightorig : $height;
		$width = ($height > $heightorig) ? $widthorig : ($widthorig / $heightorig) * $height;
	}
	
	$extn = explode(".", $ffile);
	$i = count($extn);
	$ext = strtolower($extn[$i-1]);
	
	// Resample
	$imagep = imagecreatetruecolor($width, $height);
	if ($ext == "jpg" || $ext == "jpeg" || $ext == "pjpeg") $image=imagecreatefromjpeg($ffile);
	if ($ext == "png") $image = imagecreatefrompng($ffile);
	if ($ext == "gif") $image = imagecreatefromgif($ffile);
	imagecopyresampled($imagep, $image, 0, 0, 0, 0, $width, $height, $widthorig, $heightorig);
	
	// Output
	header('Content-type: image/png');
	imagepng($imagep);
}

/**
 * Makes SEF URL
 *
 * @param string $string
 * @return string
 */
function AppleMakeSef($string) {
	if (settings::get('sef') == 'off') return $string;
	$url = settings::get('url').'/';
	
	$uri = urldecode(substr($_SERVER['REQUEST_URI'], strlen(settings::get('sub_directory'))));
	$uri = str_replace('%20', ' ', $uri);
	if (strstr($uri, 'admin') && !strstr($string, $url)) {
		$url .= 'admin/';
	}
	if ($string == 'index.php') return $url;
	
	$string = preg_replace("/((^(.*)(\/))?(index\.php))/", null, $string, 1);
	
	if (strstr($string, $url)) {
		$temp = explode($url, $string);
		if (!ereg("^\?", $temp[1])) return $string;
	}
	else if (!ereg("^\?", $string)) return $string;
	
	$string = str_replace('&amp;', '&', $string);
	$string = explode('?', $string);
	$string = explode('&', $string[1]);
	$count = count($string);
	$part = array();
	
	for ($i=0; $i<$count; $i++) {
		$parts = explode('=', $string[$i]);
		$part[strtolower($parts[0])] = $parts[1];
	}
	
	if ($part['com'] == 'content') {
		require_once(Apple_root_path.'/components/content/content.class.php');
		$return = content::getPermaLink($part);
	}
	else {
		$return = (empty($part['com']) ? settings::get('default_component') : $part['com']).'/';
		foreach ($part as $k => $v) {
			if ($k != 'com') $return .= $k.'/'.$v.'/';
		}
	}
	
	$return = str_replace(settings::get('default_component').'/', '', $return);
	$return = preg_replace("/((page\/1(\/)|page\/1(\/)?$))/", "", $return);
	$return = preg_replace("/((lng\/".DEFAULT_LANGUAGE."(\/)|lng\/".DEFAULT_LANGUAGE."(\/)?$))/", "", $return);
	//$return = preg_replace("/((lng\/".DEFAULT_LANGUAGE.")(\/)?)/", "", $return);
	
	if (!strstr($return, $url)) $return = "$url$return";
	return $return;
}

function AppleSefReverse($parts=null) {
	if ($parts == null) $parts = AppleParseURI();
	
	if (!is_array($parts)) return $parts;
	
	if ($parts[0] == 'lng') {
		$_GET['lng'] = $_REQUEST['lng'] = $parts[1];
		$_SERVER['QUERY_STRING'] = 'lng='.$parts[1];
		return $_SERVER['QUERY_STRING'];
	}
	
	$_GET['com'] = $_REQUEST['com'] = $parts[0];
	$_SERVER['QUERY_STRING'] = 'com='.$parts[0];
	for ($i=1; $i<count($parts); $i++) {
		if (trim($parts[$i]) != '') {
			$_GET[$parts[$i]] = $_REQUEST[$parts[$i]] = $parts[++$i];
			$_SERVER['QUERY_STRING'] .= '&'.$parts[$i-1].'='.$parts[$i];
		}
	}
	return $_SERVER['QUERY_STRING'];
}

function AppleParseURI($uri=null) {
	if (preg_match('/(\b)GLOBALS|_REQUEST|_SERVER|_ENV|_COOKIE|_GET|_POST|_FILES|_SESSION(\b)/i', $_SERVER['REQUEST_URI']) > 0) return ;
	if ($uri == null) $uri = $_SERVER['REQUEST_URI'];
	$uri = str_replace('%20', ' ', $uri);
	$uri = urldecode(substr($uri, strlen(settings::get('sub_directory'))));
	$uri = preg_replace("/^(\/.*)/U", '',$uri);
	$uri = str_replace('index.php', '', $uri);
	if (strstr($uri, 'admin')) $uri = substr($uri, strlen('admin/'));
	if (preg_match("/^(\?.*)/U", $uri)) return substr($uri, 1);
	
	$parts = explode('/', $uri);
	
	while (trim($parts[0]) == '' && !empty($parts))
		array_shift($parts);
	
	return $parts;
}

function AppleIsSef($uri=null) {
	if ($uri == null) $uri = $_SERVER['REQUEST_URI'];
	$uri = str_replace('%20', ' ', $uri);
	$uri = urldecode(substr($uri, strlen(settings::get('sub_directory'))));
	$uri = preg_replace("/^(\/.*)/U", '',$uri);
	$uri = str_replace('index.php', '', $uri);
	if (strstr($uri, 'admin')) $uri = substr($uri, strlen('admin/'));
	return !preg_match("/^(\?.*)/U", $uri);
}

// Checks if the current HTTP-Request is an Ajax-Request
function AppleAjaxRequest() {
	return (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && $_SERVER['HTTP_X_REQUESTED_WITH'] == "XMLHttpRequest");
}

function AppleInstalled() {
	global $database;
	$database->setQuery("SELECT * FROM `#__settings` WHERE `key`='url'");
	$r = $database->loadObject();
	return !empty($r);
}

function AppleDie($message, $title='Apple &rsaquo; ERROR!') {
	$x = substr(dirname($_SERVER['SCRIPT_FILENAME']), strlen(Apple_root_path));
	$a = explode('/', $x);
	$r = count($a);
	$r--;
	$a = '';
	for ($i=0; $i<$r; $i++) {
		$a .= '../';
	}
	die('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
	<html xmlns="http://www.w3.org/1999/xhtml">
	<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>'.$title.'</title>
	<link rel="shortcut icon" href="images/favicon.png" type="image/x-icon">
	<link rel="stylesheet" type="text/css" href="'.$a.'kernel/css/style.css">
	</head>
	<body>
	<div class="logo"></div>
	'.$message.'
	</body>
	</html>');
}
?>