<?php
/**
 * @package Apple
 * @author Parsiserve Inc.
 * @copyright Parsiserve Inc. http://www.parsiserve.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see license.txt
 * Apple is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; version 2 of the License.
 */

defined('ACCESS_ALLOW') or die('<H1>Forbidden</H1>You are not allowed to be here<hr>');

class AppleHTML {
	
	/**
	 * Generates publish icon
	 *
	 * @param int $id
	 * @param int $published  set 1 if item is published otherwise 0
	 * @return string
	 */
	public function publishIcon($id, $published, $url=null) {
		if ($url == null) {
			$url = '?';
			if (($x=AppleGetParam('com', $_GET)) != null) $url .= 'com='.$x;
			if (($x=AppleGetParam('section', $_GET)) != null) $url .= '&'.'section='.$x;
			if (($x=AppleGetParam('page', $_GET)) != null) $url .= '&'.'page='.$x;
			if (($x=AppleGetParam('lng', $_GET)) != null) $url .= '&'.'lng='.$x;
		}
		
		if ($published == 1) {
			return '<a href="javascript:void(0);" onclick="location.href=\''.AppleMakeSef($url.'&task=unpublish&id='.$id).'\'"><img src="images/published.gif" alt="'.__('Published').'" title="'.__('Published').'"></a>';
		}
		return '<a href="javascript:void(0);" onclick="location.href=\''.AppleMakeSef($url.'&task=publish&id='.$id).'\'"><img src="images/unpublished.gif" alt="'.__('Unpublished').'" title="'.__('Unpublished').'"></a>';
	}
	
	public function moveUpIcon($id, $condition=true, $task='reorder') {
		if ($condition) {
			$url = '?'.AppleClearMessage(AppleSefReverse());
			return '<a href="javascript:void(0);" onclick="location.href=\''.$url.'&task='.$task.'&do=up&id='.$id.'\'"><img src="images/move_up.gif" alt="'.__('up').'" title="'.__('up').'"></a>';
		}
		return '&nbsp;';
	}
	
	public function moveDownIcon($id, $condition=true, $task='reorder') {
		if ($condition) {
			$url = '?'.AppleClearMessage(AppleSefReverse());
			return '<a href="javascript:void(0);" onclick="location.href=\''.$url.'&task='.$task.'&do=down&id='.$id.'\'"><img src="images/move_down.gif" alt="'.__('down').'" title="'.__('down').'"></a>';
		}
		return '&nbsp;';
	}
	
	public function editIcon($id) {
		$url = '?'.AppleClearMessage(AppleSefReverse());
		return '<a href="'.$url.'&task=edit&id='.$id.'"><img src="images/edit.png" style="border:0;" title="'.__('Edit').'" alt="'.__('Edit').'"></a>';
	}
	
	public function deleteIcon($id, $tb=null) {
		$url = '?'.AppleClearMessage(AppleSefReverse());
		return '<a href="javascript:void(0)" onclick="if (confirm(\''.__('Are you sure you want to delete selected item?').'\')) '.($tb != null ? 'go2(\''.$url.'&task=delete&id='.$id.'&tb='.$tb.'\', \'message\')' : 'window.location.href=\''.$url.'&task=delete&id='.$id.'\'').'"><img src="images/delete.png" style="border:0;" title="'.__('Delete').'" alt="'.__('Delete').'"></a>';
	}
	
	public function makeOption($value, $text=null) {
		$option = new stdClass();
		$option->value = $value;
		$option->text = $text != null ? $text : $value;
		return $option;
	}
	
	public function makeText($name, $value=null, $extra=null, $required=false) {
		return '<input type="text" name="'.$name.'" class="input" value="'.$value.'" '.($required ? 'required="true" ' : '').$extra.' />';
	}
	
	public function makeHiddenText($name, $value=null, $extra=null) {
		return '<input type="hidden" name="'.$name.'" class="input" value="'.$value.'" '.$extra.' />';
	}
	
	public function makePassword($name='password', $extra=null, $required=true) {
		return '<input type="password" name="'.$name.'" class="input" '.($required ? 'required="true" ' : '').$extra.' dir="ltr" />';
	}
	
	public function makeTextArea($name, $value=null, $extra=null, $required=false) {
		return '<textarea name="'.$name.'" class="input" '.($required ? 'required="true" ' : '').$extra.'>'.$value.'</textarea>';
	}
	
	/**
	 *  Generates an HTML radio buttons
	 *
	 * @param array $options
	 * @param string $name
	 * @param string $extra
	 * @param unknown_type $checked
	 * @param string $value
	 * @param string $text
	 * @return string HTML
	 */
	public function makeRadio($options, $name, $extra=null, $checked=null, $value='value', $text='text') {
		@reset($options);
		if (empty($options) || !is_array($options)) return ;
		$out = null;
		foreach ($options as $option) {
			$check = ($option->$value == $checked) ? 'checked="checked"' : '';
			$out .= '<label for="'.$option->$value.'"><input type="radio" name="'.$name.'" id="'.$option->$value.'" value="'.$option->$value.'" '.$check.' '.$extra.' />'.$option->$text.'</label>&nbsp;&nbsp;';
		}
		return $out."\n";
	}
	
	/**
	 *  Generates an HTML checkbox buttons
	 *
	 * @param array $options
	 * @param string $name
	 * @param string $extra
	 * @param unknown_type $checked
	 * @param string $value
	 * @param string $text
	 * @return string HTML
	 */
	public function makeCheckbox($options, $name, $extra=null, $checked=null, $value='value', $text='text') {
		@reset($options);
		if (empty($options) || !is_array($options)) return ;
		$out = null;
		foreach ($options as $option) {
			$check = ($option->$value == $checked) ? 'checked="checked"' : '';
			$out .= '<label for="'.$option->$value.'"><input type="checkbox" name="'.$name.'" id="'.$option->$value.'" value="'.$option->$value.'" '.$check.' '.$extra.' />'.$option->$text.'</label>&nbsp;&nbsp;';
		}
		return $out."\n";
	}
	
	/**
	 *  Generates an HTML list
	 *
	 * @param array $options
	 * @param string $name
	 * @param string $extra
	 * @param unknown_type $selected
	 * @param string $value
	 * @param string $text
	 * @return string HTML
	 */
	public function makeList($options, $name, $extra=null, $selected=null, $value='value', $text='text') {
		@reset($options);
		$out = null;
		$out .= '<select name="'.$name.'" '.$extra.'>'."\n";
		if (empty($options) || !is_array($options)) return ;
		foreach ($options as $option) {
			$select = null;
			if (is_array($selected) && in_array($option->$value, $selected)) $select = 'selected = "selected"';
			else if ($option->$value == $selected) $select = 'selected = "selected"';
			
			$out .= '<option value="'.$option->$value.'" '.$select.'>'.$option->$text.'</option>'."\n";
		}
		$out .= '</select>';
		return $out;
	}
	
	public function makeButton($value, $extra=null) {
		return '<input type="button" class="button" value="'.$value.'" '.$extra.'>';
	}
	
	public function makeSubmitButton($name='save', $extra=null) {
		return '<input type="submit" class="button" name="'.$name.'" id="'.$name.'" '.$extra.' value="'.__('Send').'" />';
	}
	
	public function makeResetButton($extra=null) {
		return '<input type="reset" class="button" '.$extra.' value="'.__('Reset').'" />';
	}
	
	public function makeCaptcha($url) {
		return '<img id="img_captcha" src="'.AppleMakeSef($url).'" /> <a href="javascript:void(0)" onclick="playCaptcha(\''.$url.'\')"><img src="images/accessibility.png" alt="Accessibility" title="Accessibility" /></a>';
	}
}

class table {
	private $class, $style, $id, $cellpadding, $cellspacing, $extra;
	private $TR = 0;
	private $TR_ATT = array();
	private $TH = array();
	private $TD = array();
	public $sortable = false;
	public $sort_url = null;
	
	public function __construct($class=null, $id=null, $style=null, $cellpadding=0, $cellspacing=0, $extra=null) {
		$this->class = $class;
		$this->id = $id;
		$this->style = $style;
		$this->cellpadding = $cellpadding;
		$this->cellspacing = $cellspacing;
		$this->extra = $extra;
	}
	
	public function addTR($extra=null) {
		$this->TR++;
		$this->TR_ATT[$this->TR]->extra = $extra;
	}
	
	public function addTH($value, $style=null, $extra=null) {
		$c = count($this->TH[$this->TR]);
		if ($c == 0) $style .= 'border-'.(settings::get('direction') == 'rtl' ? 'right' : 'left').':0;';
		$this->TH[$this->TR][$c]->style = $style;
		$this->TH[$this->TR][$c]->extra = $extra;
		$this->TH[$this->TR][$c]->value = $value;
	}
	
	public function addTD($value, $style=null, $extra=null) {
		$c = count($this->TD[$this->TR]);
		if ($c == 0) $style .= 'border-'.(settings::get('direction') == 'rtl' ? 'right' : 'left').':0;';
		$this->TD[$this->TR][$c]->style = $style;
		$this->TD[$this->TR][$c]->extra = $extra;
		$this->TD[$this->TR][$c]->value = $value;
	}
	
	public function show() {
		$show = null;
		$t = false;
		if ($this->sortable) $show.= '<div id="sort_button" style="text-align:center;width:100%;padding-bottom:8px;direction:rtl;"></div>';
		$show .= "\n<table class=\"$this->class".($this->sortable ? " sortable" : "")."\" id=\"$this->id\" style=\"$this->style\" cellpadding=\"$this->cellpadding\" cellspacing=\"$this->cellspacing\" dir=\"rtl\" $this->extra>\n";
		for ($i=1; $i<=$this->TR; $i++) {
			if (empty($this->TH[$i]) && !$t) {
				$show .= "<tbody".($this->sortable?" id=\"tb_$this->id\"":"").">\n";
				$t = true;
			}
			
			$show .= '<tr '.(empty($this->TH[$i]) ? 'class="'.($i%2 == 0 ? 'even' : 'odd').'" ' : '').$this->TR_ATT[$i]->extra.'>';
			if (!empty($this->TH[$i])) {
				foreach ($this->TH[$i] as $TH) {
					$show .= "<th style=\"$TH->style\" $TH->extra>$TH->value</th>\n";
				}
			}
			
			if (!empty($this->TD[$i])) {
				foreach ($this->TD[$i] as $TD) {
					$show .= "<td style=\"$TD->style\" $TD->extra>$TD->value</td>\n";
				}
			}
			$show .= "</tr>\n";
		}
		if ($t) $show .= "</tbody>\n";
		$show .= "</table>\n";
		if ($this->sortable) $show .= '<div id="sort_info" style="display:none;"></div>
		<script type="text/javascript">
		//<![CDATA[
		Sortable.create("tb_'.$this->id.'", {tag:\'tr\', treeTag:\'table\', tree:\'true\', scroll:window, onUpdate:function(){$(\'sort_info\').innerHTML = Sortable.serialize(\'tb_'.$this->id.'\');}, onChange:function(){AppleList.reColor(\''.$this->id.'\'); $(\'sort_button\').innerHTML = "<a href=\"javascript:void(0);\" onclick=\"AppleReOrder(\''.$this->sort_url.'\', \'message\');\" class=\"reordering_button\">'.__('Save New Ordering').'</a>";}});
		//]]>
		</script>';
		unset($this->TH);
		unset($this->TD);
		echo $show;
	}
}
?>